'use strict'

exports.__esModule = true
exports.default = getChunkScripts
exports.getRelRegex = void 0

var _url = _interopRequireDefault(require('url'))

var _path = _interopRequireDefault(require('path'))

var _findChunks = _interopRequireWildcard(require('./findChunks'))

function _interopRequireWildcard(obj) {
  if (obj && obj.__esModule) {
    return obj
  } else {
    var newObj = {}
    if (obj != null) {
      for (var key in obj) {
        if (Object.prototype.hasOwnProperty.call(obj, key)) {
          var desc =
            Object.defineProperty && Object.getOwnPropertyDescriptor
              ? Object.getOwnPropertyDescriptor(obj, key)
              : {}
          if (desc.get || desc.set) {
            Object.defineProperty(newObj, key, desc)
          } else {
            newObj[key] = obj[key]
          }
        }
      }
    }
    newObj.default = obj
    return newObj
  }
}

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {default: obj}
}

const isExternalDefault = (regex, chunkName, mod) =>
  mod &&
  mod.providedExports !== null &&
  mod.providedExports.indexOf('default') > -1 &&
  mod.identifier.indexOf(chunkName) > -1 &&
  regex.test(mod.identifier) &&
  (mod.issuerName === null ||
    mod.identifier.indexOf(
      _path.default.dirname(mod.issuerName).replace(/^\./, '')
    ) === -1)

const reCache = {},
  relativePkg = /^\.\//

const getRelRegex = chunkName => {
  if (!reCache[chunkName])
    reCache[chunkName] = new RegExp(
      `/${chunkName.replace(relativePkg, '')}((/index)*\.(m?jsx?|tsx?))+`
    )
  return reCache[chunkName]
}

exports.getRelRegex = getRelRegex

const isRelativeDefault = (relRegex, chunkName, mod) =>
  mod &&
  mod.identifier.indexOf(chunkName) > -1 &&
  mod.providedExports.indexOf('default') > -1 &&
  relRegex.test(mod.identifier)

function getChunkScripts(stats, cache, {async = true, defer, preload}) {
  let scripts = [],
    preloads = [],
    resolve = fn => _url.default.resolve(stats.publicPath, fn),
    preloadAttrs = ''

  function _ref(k) {
    return `${k}="${preload[k]}"`
  }

  if (typeof preload === 'object') {
    preloadAttrs = Object.keys(preload)
      .map(_ref)
      .join(' ')
    preloadAttrs = ` ${preloadAttrs}`
  }

  let i,
    j,
    k,
    moduleIds = {},
    chunkNames = cache.getChunkNames()

  for (let chunk of (0, _findChunks.default)(stats, chunkNames)) {
    for (i = 0; i < chunk.files.length; i++) {
      const file = chunk.files[i],
        filename = resolve(file),
        rbNames = []

      for (j = chunkNames.length - 1; j > -1; j--) {
        const chunkName = chunkNames[j],
          modRegex = (0, _findChunks.getRegex)(chunkName),
          relRegex = getRelRegex(chunkName)

        for (k = 0; k < chunk.modules.length; k++) {
          const mod = chunk.modules[k]

          if (
            isRelativeDefault(relRegex, chunkName, mod) ||
            isExternalDefault(modRegex, chunkName, mod)
          ) {
            moduleIds[chunkName] = mod.id
            chunkNames.splice(j, 1)
            rbNames.push(chunkName)
            break
          }
        }
      }

      if (preload) {
        const p = `<link rel="preload" as="script" href="${filename}"${preloadAttrs}>`
        if (chunk.entry || chunk.initial) preloads.unshift(p)
        else preloads.push(p)
      }

      scripts.push(
        `<script` +
          `${
            rbNames.length > 0
              ? ` data-rb="${rbNames.join('+')}"`
              : ' data-rb=""'
          } ` +
          `src="${filename}" ` +
          `${defer ? 'defer ' : async ? 'async ' : ''}` +
          `onload="this.setAttribute('data-loaded', 'true')"` +
          `></script>`
      )
    }
  }

  scripts.unshift(
    `<script id="__INITIAL_BROKER_CHUNKS__" type="application/json">${JSON.stringify(
      moduleIds
    )}</script>`
  )
  preloads = preloads.join('')
  scripts = scripts.join('')
  return {
    scripts,
    preload: preloads,
    toString: () => preloads + scripts,
  }
}
